from cpython cimport array 
import array 
from scipy.optimize import minimize_scalar
from numpy import zeros
import numpy as np
def RS(d): #rescale distribution d
    s=np.std(d)
    m=np.average(d)
    if s:
        return (d-m)/s
    else:
        return d-m 
def EMDt(double [:] dd1,double [:] d2, double t): # compute EMD between distributions dd1 and d2+t. In the case of spectra the distribution is a taken to be a collection of point masses resulting in a cummulative distribution that is a piecewise constant i.e. is a increasing step function. 
    cdef int ii
    cdef int l1=len(dd1)
    cdef double [:] d1=dd1.copy()
    for ii in range(l1):
        d1[ii]+=t
    cdef int l2=len(d2)
    cdef int i=0
    cdef int j=0
    cdef double a=0
    cdef double s1=0
    cdef double s2=0
    cdef double p1=1.0/l1
    cdef double p2=1.0/l2
    cdef double st=min(d1[0],d2[0])
    while i<l1 and j<l2: #keep track of steps in the CDFs.
        if d1[i]<=d2[j]:
            a+=abs((s1-s2)*(d1[i]-st))
            st=d1[i]
            i+=1
            s1+=p1
        else:
            a+=abs((s1-s2)*(d2[j]-st))
            st=d2[j]
            j+=1
            s2+=p2
    if i==l1: #What to do when either of the distributions ends.
        while j<l2:
            a+=abs((1-s2)*(d2[j]-st))
            st=d2[j]
            j+=1
            s2+=p2
    elif j==l2:
        while i<l1:
            a+=abs((s1-1)*(d1[i]-st))
            st=d1[i]
            i+=1
            s1+=p1        
    return a

def optEMD(d1,d2): #find t that minimizes EMD between d1 and d2 and return EMD*=EMD(d1,d2+t)
    return minimize_scalar(lambda tt: EMDt(d1,d2,tt),method='brent',tol=0.001, options={'maxiter': 50}).fun

